
# FUNKCIJE

# Fukcije u R-u su objekti koji primaju ulazne vrijednosti
# i vracaju izlazne. Ustvari, sve sto se dogada u R-u dogada 
# se preko funkcija, cak i postavljanje varijabli, petlje, grananja, 
# operatori i slicno. Kao primjer, evo upotrebe 
# operatora '+' u smislu funkcije 
# (dakako, ne ne bi bilo razumno korisiti ga na taj nacin u praksi):

'+'(2,3)
# [1] 5

help("function")

# Definicija funkcije je jednostavna i slijedi opci oblik:

# moja_funkcija <- function(arg1, arg2,...) 
# { 
#  izraz(i);
# return(objekt)
# } 


# gdje su '*moja_funkcija*' ime funkcije, 
# '*function*' - kljucna rijec (naredba) 
# programskg jezika R, argumenti '*arg1*', '*arg2*', 
# i tako dalje, su formalni parametri ulaznih podaka, 
# viticaste zagrade '{}' omedjuju tijelo funkcije, 
# tj. izraze/naredbe koje se u funkciji izvrsavaju.

# Ako nema ulaznih podataka, onda se argumenti izostavljaju, 
# ali ne i okrugle ('()') zagrade. 
# Medutim, ako imamo samo jedan izraz/naredbu u tijelu funkcije, 
# onda se viticaste zagrade ('{}') mogu 
# izostaviti (R-interpreter ne javlja pogresku ako su ostavljene).

# Poziv funkcije ostaruje se upisom njenog imena i stvarnih 
# argumenata na mjestu formalnih parametara. 
# U slucaju da funkcija nema argumente u pozivu 
# funkcije se pise prazne okrugle zagrade 
# uz ime funkcije, npr. moja_funkcija().

# Funkcije mogu imati i pretpostavljene vrijednosti 
# argumenata (sa sintaksom 'arg=vrijednost', pa se u 
# slucaju poziva uzimaju pretpostavljene vrijednosti, 
# ako se argument ne navede. Evo primjera:

moja_funkcija <- function(a, b = 3) {
    a^2 + b^2  ##vrati zbroj kvadrata
}

moja_funkcija(1, 2)

## [1] 5

moja_funkcija(1)

## [1] 10

# Dok je u prvom slucaju vrijednost argumenta 'b' bila 2, 
# pa je 12 +22 dalo 5, u drugom slucaju poziva funkcije, 
# drugi argument nije naveden, pa je rezultat dobiven 
# koristeci pretpostavljenu vrijednost za parametar b (3) sto 
# je dalo rezultat 10, prema: 12 +32.

# Za vracanje (izlaznih) vrijednosti iz funkcije, 
# moze se koristiti kljucna rijec/naredba 'return'.

# Ako se to ne nacini, kao sto je to u prethodnom 
# primjeru vidljivo, onda se vraca vrijednost koju 
# daje zadnji izraz tijela funkcije.

# Buduci da su i funkcije objekti, moguce je bilo koju 
# funkciju proslijediti kao parametar/argument druge funkcije. 
# Kao primjer moze se uzeti sapply() funkcija koja 
# uzima na primjer nasu funkciju moja_funkcija() kao 
# argument s kojim djeluje na clanove nekog polja ili liste, 
# zadanih u prvom argumentu.

help("sapply")

lista <- c(1, 2, 3)
sapply(lista, moja_funkcija)

## [1] 10 13 18

#################################
#################################

# Primjer
# Napisati funkciju koja se zove "Ekstremna_Vrijednost" 
# i koja prima 3 ulazna argumenta od kojih je jedan 
# obavazan a dva proizvoljna, kako slijedi:
# 1. argument: vektor x, proizvoljne duljine
# 2. argument: opcionalni argument, ime argumenta "maksimum"; 
#              izvorno postavljena vrijednost ("default" vrijednost) je
#             maksimum=TRUE 
# 3. argument: opcionalni argument, ime argumenta "raspon";
#              izvorno postavljena vrijednost ("default" vrijednost) je
#             raspon=TRUE
# IZLAZ iz funkcije ovisi o vrijednostima opciponalnih argumenata, kako slijedi:
#
# A) maksimum=TRUE,raspon=TRUE: izlaz je lista koja sadrzi 
# (Najmanji element vektora, 
# najveci element vektora, 
# raspon brojeva u kojem se nalaze elementi vektora)

# B) maksimum=TRUE,raspon=FALSE: izlaz je lista koja sadrzi 
# (Najmanji element vektora, 
# najveci element vektora)

# C) maksimum=FALSE,raspon=FALSE: izlaz je lista koja sadrzi 
# (Najmanji element vektora)

# D) maksimum=FALSE,raspon=TRUE: izlaz je lista koja sadrzi 
# (Najmanji element vektora, 
# raspon brojeva u kojem se nalaze elementi vektora)

# Napomena: u rjesenju se ne smiju koristiti ugradjene R-ove funcije min i max

Ekstremna_Vrijednost<-function(x,maksimum=TRUE,raspon=TRUE){
  TrenutniMin<-x[1]; 
  
  if (maksimum | raspon){
    TrenutniMax<-x[1];
    for (i in x){
      if (i<TrenutniMin){
        TrenutniMin<-i
      }else{
        if (i>TrenutniMax){TrenutniMax<-i}
      }
    };
    if (maksimum & raspon){rezultat<-list(NajmanjiElement=TrenutniMin, 
                         NajveciElement=TrenutniMax,
                         MaxRaspon=TrenutniMax-TrenutniMin)}
    if (maksimum & !raspon){rezultat<-list(NajmanjiElement=TrenutniMin, 
                         NajveciElement=TrenutniMax)}
    if (!maksimum & raspon){rezultat<-list(NajmanjiElement=TrenutniMin, 
                         MaxRaspon=TrenutniMax-TrenutniMin)}
  }else{
    for(i in x){
    if (i<TrenutniMin){TrenutniMin<-i}
    };
    rezultat<-list(NajmanjiElement=TrenutniMin)
  }
  return(rezultat)
}

v<-c(-3,4,-7,5,10);
Ekstremna_Vrijednost(v)
Ekstremna_Vrijednost(v, TRUE, FALSE)
Ekstremna_Vrijednost(v, FALSE, FALSE)
Ekstremna_Vrijednost(v, FALSE, TRUE)

